<?php
define('ELVANTOTOOLS_ROSTER_FIELD_NAMES','firstname,lastname,preferred_name,firstinitial,lastinitial');
define('ELVANTOTOOLS_ROSTER_DEFAULT_COLUMNS','firstname,lastinitial');
define('ELVANTOTOOLS_ROSTER_DEFAULT_WIDGET_COLUMNS','firstname,lastinitial');
define('ELVANTOTOOLS_SONGS_FIELD_NAMES','title,artist,ccli_number,album,arrangement_title,copyright,first_line,video_link,video_embed,audio_link,audio_sample,image_link');
define('ELVANTOTOOLS_SONGS_DEFAULT_COLUMNS','audio_sample,audio_link,video_link,ccli_number,title,artist,copyright,first_line,video_embed');
define('ELVANTOTOOLS_SONGS_DEFAULT_WIDGET_COLUMNS','title');

class ElvantoToolsSettingsPage
{
    /**
     * Holds the values to be used in the fields callbacks
     */
    private $options;

    private	$roster_field_names;
	private $songs_field_names;
	
	public function GetRosterFields() { 
		return $roster_field_names;
	}

	public function GetSongsFields() { 
		return $songs_field_names;
	}

    /**
     * Start up
     */
    public function __construct()
    {
        add_action( 'admin_menu', array( $this, 'add_plugin_page' ) );
        add_action( 'admin_init', array( $this, 'page_init' ) );
		$this->roster_field_names = explode(',',ELVANTOTOOLS_ROSTER_FIELD_NAMES);
		$this->songs_field_names = explode(',',ELVANTOTOOLS_SONGS_FIELD_NAMES);
    }

    /**
     * Add options page
     */
    public function add_plugin_page()
    {
        // This page will be under "Settings"
        $page = add_options_page(
            'Settings Admin', 
            'ElvantoTools', 
            'manage_options', 
            'elvantotools-setting-admin', 
            array( $this, 'create_admin_page' )
        );
		add_action( 'admin_print_styles-' . $page, array( $this,'elvantotools_admin_styles' ));		
    }

	public function elvantotools_admin_styles() {
       /*
        * It will be called only on your plugin admin page, enqueue our stylesheet here
        */
       wp_enqueue_style('elvantotoolsStylesheet');
   }
    /**
     * Options page callback
     */
    public function create_admin_page()
    {
        // Set class property
        $this->options = get_option( 'elvantotools_options' );
		//error_log(print_r($this->options, true));
        ?>
        <script type="application/javascript">
		function toggleSection(e) {
			var toggle = jQuery(e);
			if (toggle.text() == "+") {
				toggle.parent().next().next().slideUp("fast", function() { jQuery(this).prev().slideUp(); });
				toggle.text("-");
			} else {
				toggle.parent().next().slideDown("fast", function() { jQuery(this).next().slideDown(); });
				toggle.text("+");
			}
		}
		jQuery(function(){
			jQuery('div.wrap div.elvantotools-expand:not(:first)').text('-');
			jQuery('div.wrap div.elvantotools-section-header:not(:first)').hide();			
			jQuery('div.wrap table.form-table:not(:first)').hide();			
		});
		</script>
        <div class="wrap">
        	<div class="elvantotools-settings-header">
            	<div class="elvantotools-settings-header-logo">
                	<a href="https://elvantotools.me"><img src="<?=plugins_url('ElvantoToolsPPHeader.png', __FILE__)?>"></a>
                </div>
                <div class="elvantotools-settings-header-text">
                    <h2>Shortcode Settings</h2>
                    <p>Add a roster to any post or page by including the text <code>[elvanto_roster]</code></p>
                    <p>Add a song list to any post or page by including the text <code>[elvanto_songs]</code></p>
                </div>
            </div>
            <form method="post" action="options.php">
            <?php
                // This prints out all hidden setting fields
                settings_fields( 'elvantotools_option_group' );   
                do_settings_sections( 'elvantotools-setting-admin' );
                submit_button(); 
            ?>
            </form>
        </div>
        <?php
    }

	public function section_title($text) {
		return '<div class="elvantotools-expand" onClick="toggleSection(this);">+</div>' . $text;
	}
	
    /**
     * Register and add settings
     */
    public function page_init()
    {        
 		 wp_register_style('elvantotoolsStylesheet', plugins_url('style.css', __FILE__) );
         register_setting(
            'elvantotools_option_group', // Option group
            'elvantotools_options', // Option name
            array( $this, 'sanitize' ) // Sanitize
        );

        add_settings_section(
            'setting_section_defaults', // ID
            $this->section_title('Defaults'), // Title
            array( $this, 'print_section_defaults' ), // Callback
            'elvantotools-setting-admin' // Page
        );  

        add_settings_section(
            'setting_section_roster', // ID
             $this->section_title('Roster'), // Title
            array( $this, 'print_section_roster' ), // Callback
            'elvantotools-setting-admin' // Page
        );  

        add_settings_section(
            'setting_section_songs', // ID
             $this->section_title('Songs'), // Title
            array( $this, 'print_section_songs' ), // Callback
            'elvantotools-setting-admin' // Page
        );  

        add_settings_field(
            'elvanto_api_key', // ID
            'Elvanto API Key', // Title 
            array( $this, 'elvanto_api_key_callback' ), // Callback
            'elvantotools-setting-admin', // Page
            'setting_section_defaults' // Section           
        );      

        add_settings_field(
            'elvanto_weeks', // ID
            'Weeks ahead', // Title 
            array( $this, 'elvanto_weeks_callback' ), // Callback
            'elvantotools-setting-admin', // Page
            'setting_section_defaults' // Section           
        );      

        add_settings_field(
            'elvanto_services', // ID
            'Services ahead', // Title 
            array( $this, 'elvanto_services_callback' ), // Callback
            'elvantotools-setting-admin', // Page
            'setting_section_defaults' // Section           
        );      

        add_settings_field(
            'song_video_width', // ID
            'Video width', // Title 
            array( $this, 'song_video_width_callback' ), // Callback
            'elvantotools-setting-admin', // Page
            'setting_section_songs' // Section           
        );      

        add_settings_field(
            'song_video_height', // ID
            'Video height', // Title 
            array( $this, 'song_video_height_callback' ), // Callback
            'elvantotools-setting-admin', // Page
            'setting_section_songs' // Section           
        );      
		
        add_settings_field(
            'elvanto_date_format', // ID
            'Date format', // Title 
            array( $this, 'elvanto_date_format_callback' ), // Callback
            'elvantotools-setting-admin', // Page
            'setting_section_defaults' // Section           
        );      

        add_settings_field(
            'elvanto_time_zone', // ID
            'Timezone', // Title 
            array( $this, 'elvanto_time_zone_callback' ), // Callback
            'elvantotools-setting-admin', // Page
            'setting_section_defaults' // Section           
        );      

        add_settings_field(
            'roster_service_types', // ID
            'Service Types', // Title 
            array( $this, 'roster_service_types_callback' ), // Callback
            'elvantotools-setting-admin', // Page
            'setting_section_roster' // Section           
        );      

		add_settings_field(
				'roster_show_depts', 
				'Show departments', 
				array( $this, 'roster_show_depts_callback' ), 
				'elvantotools-setting-admin', 
				'setting_section_roster',
				array('field'=>'roster_show_depts')
		);      

		add_settings_field(
				'song_in_plan', 
				'Songs in the plan', 
				array( $this, 'song_in_plan_callback' ), 
				'elvantotools-setting-admin', 
				'setting_section_songs',
				array('field'=>'song_in_plan')
		);      

		add_settings_field(
				'roster_show_subs', 
				'Show sub-departments', 
				array( $this, 'roster_show_subs_callback' ), 
				'elvantotools-setting-admin', 
				'setting_section_roster',
				array('field'=>'roster_show_subs')
		);      
		
        add_settings_field(
            'roster_departments', // ID
            'Departments', // Title 
            array( $this, 'roster_departments_callback' ), // Callback
            'elvantotools-setting-admin', // Page
            'setting_section_roster' // Section           
        );      

        add_settings_field(
            'songs_service_types', // ID
            'Service Types', // Title 
            array( $this, 'songs_service_types_callback' ), // Callback
            'elvantotools-setting-admin', // Page
            'setting_section_songs' // Section           
        );      

        add_settings_field(
            'songs_categories', // ID
            'Categories', // Title 
            array( $this, 'songs_categories_callback' ), // Callback
            'elvantotools-setting-admin', // Page
            'setting_section_songs' // Section           
        );      

		foreach ($this->roster_field_names as $field) {
			add_settings_field(
				'roster_'.$field, 
				ucwords(str_replace('_',' ',$field)), 
				array( $this, 'roster_callback' ), 
				'elvantotools-setting-admin', 
				'setting_section_roster',
				array('field'=>$field)
			);      
		}

		foreach ($this->songs_field_names as $field) {
			add_settings_field(
				'songs_'.$field, 
				ucwords(str_replace('_',' ',$field)), 
				array( $this, 'songs_callback' ), 
				'elvantotools-setting-admin', 
				'setting_section_songs',
				array('field'=>$field)
			);      
		}
		
    }

    /**
     * Sanitize each setting field as needed
     *
     * @param array $input Contains all settings fields as array keys
     */
    public function sanitize( $input )
    {
		//error_log('unsanitized='.print_r($input, true));		
        $new_input = array();
		
		$t = new DateTime();
        if(isset($input['elvanto_date_format']) && $t->format($input['elvanto_date_format']) !== FALSE)
            $new_input['elvanto_date_format'] = $input['elvanto_date_format'];

		if( isset( $input['offset'] ) )
            $new_input['offset'] = absint( $input['offset'] );
		
		if( isset( $input['elvanto_weeks'] ) )
            $new_input['elvanto_weeks'] = max(absint( $input['elvanto_weeks'] ),1);

		if( isset( $input['song_video_height'] ) )
            $new_input['song_video_height'] = absint( $input['song_video_height'] );

		if( isset( $input['song_video_width'] ) )
            $new_input['song_video_width'] = absint( $input['song_video_width'] );

		if( isset( $input['elvanto_services'] ) ) {
			switch (trim($input['elvanto_services'])) {
				case '0':
				case '':
					$new_input['elvanto_services'] = '';
					break;
				default:
					$new_input['elvanto_services'] = absint( $input['elvanto_services']);
			}
		}

		foreach (array('elvanto_api_key','elvanto_time_zone','roster_service_types','roster_departments','songs_service_types','songs_categories','roster_show_depts','roster_show_subs', 'song_in_plan') as $field) {
			if( isset( $input[$field] ) )
				$new_input[$field] = sanitize_text_field( $input[$field] );
		}

		foreach ($this->roster_field_names as $name) {
			$field = 'roster_'.$name;
			if( isset( $input[$field] ) )
				$new_input[$field] = sanitize_text_field( $input[$field] );
		}

		foreach ($this->songs_field_names as $name) {
			$field = 'songs_'.$name;
			if( isset( $input[$field] ) )
				$new_input[$field] = sanitize_text_field( $input[$field] );
		}
		
		//error_log('sanitized='.print_r($new_input, true));
        return $new_input;
    }

    /** 
     * Print the Section text
     */
	 public function print_section_header($desc) {
		 print '<div class="elvantotools-section-header"><div class="elvantotools-section-header-desc">'.$desc.'</div><div class="elvantotools-section-header-save">';
		 submit_button();
		 print '</div></div>';
	 }
	 
    public function print_section_defaults()
    {
        $this->print_section_header('Default settings when using the [elvanto_roster] or [elvanto_songs] shortcodes on a page or post.<br><br>You can override these by using the shortcode attribute listed beside each setting e.g. <code>[elvanto_roster attribute="value"]</code>');
    }

    public function print_section_roster()
    { 
        $this->print_section_header('Display the following name details for each person on the roster.<br><br>You can override these by using the columns shortcode attribute e.g. <code>[elvanto_roster columns="firstname,lastinitial"]</code>');
    }

    public function print_section_songs()
    { 
        $this->print_section_header('Display the following song details for each song on the list.<br><br>You can override these by using the columns shortcode attribute e.g. <code>[elvanto_songs columns="title,author"]</code>');
    }

    /** 
     * Get the settings option array and print one of its values
     */
	public function print_input_text($type, $id, $placeholder = '', $class='elvantotools') {
        printf('<input class="%s" type="%s" id="%s" name="elvantotools_options[%s]" placeholder="%s" value="%s" />',
			$class, $type, $id, $id, $placeholder, isset( $this->options[$id] ) ? esc_attr( $this->options[$id]) : '');
	}	 

	public function print_checkbox($id, $class='elvantotools') {
        printf('<input class="%s" type="checkbox" id="%s" name="elvantotools_options[%s]" %s />',
			$class, $id, $id, isset( $this->options[$id] ) ? ($this->options[$id] == 'on' ? 'CHECKED' : '') : '');
	}

	public function print_textarea($id, $placeholder = '', $class='elvantotools') {
        printf('<textarea class="%s" id="%s" name="elvantotools_options[%s]" rows="3" placeholder="%s">%s</textarea>',
			$class, $id, $id, esc_attr($placeholder), isset( $this->options[$id] ) ? esc_attr($this->options[$id]) : '');
	}

	public function print_select($id, $values, $delimiter = '', $class='elvantotools') {
		$default = isset($this->options[$id]) ? $this->options[$id] : '';
        printf('<select class="%s" id="%s" name="elvantotools_options[%s]"><option value=""></option>', $class, $id, $id);
		$cur_group = '';
		foreach ($values as $value) {
			$att = ($value && $value == $default) ? 'SELECTED' : '';
			if ($delimiter != '') {
				$parsed = explode($delimiter, $value);
				if (count($parsed) > 1) {
					if ($parsed[0] != $cur_group) {
						if ($cur_group != '') {
							printf('</optgroup>');
						}
						$cur_group = $parsed[0];
						printf('<optgroup label="%s">', $cur_group);
					}
					$label = implode($delimiter, array_slice($parsed,1));
				} else {
					$label = $value;
				}
			} else {
				$label = $value;
			}
		    printf('<option value="%s" %s>%s</option>', $value, $att, str_replace('_',' ',$label));
		}
		if ($delimiter != '' && $cur_group != '')
			printf('</optgroup>');
		printf('</select>');
	}
	
    public function elvanto_api_key_callback() { $this->print_input_text('text', 'elvanto_api_key'); echo '&nbsp;&nbsp;Find this in your <a href="elvanto.com">Elvanto</a>&trade; settings under Integrations';} 
    public function elvanto_weeks_callback() { $this->print_input_text('text', 'elvanto_weeks'); echo '&nbsp;&nbsp;(weeks)'; } 
    public function elvanto_services_callback() { $this->print_input_text('text', 'elvanto_services'); echo '&nbsp;&nbsp;(services)'; } 
    public function song_video_height_callback() { $this->print_input_text('text', 'song_video_height'); echo '&nbsp;&nbsp;(video_height)';  } 
    public function song_video_width_callback() { $this->print_input_text('text', 'song_video_width'); echo '&nbsp;&nbsp;(video_width)'; } 
    public function elvanto_date_format_callback() { $this->print_input_text('text', 'elvanto_date_format', 'e.g. M j, Y'); echo '&nbsp;&nbsp;(dates)&nbsp;&nbsp;<a href="http://php.net/manual/en/function.date.php">valid formats</a>'; } 
    public function elvanto_time_zone_callback() { $this->print_select('elvanto_time_zone', timezone_identifiers_list(), '/'); echo '&nbsp;&nbsp;(zone)'; } 
    public function roster_service_types_callback() { $this->print_input_text('text', 'roster_service_types'); echo '&nbsp;&nbsp;(service_types)'; } 
    public function roster_show_depts_callback() { $this->print_checkbox('roster_show_depts'); echo '&nbsp;&nbsp;(show_depts)'; } 
    public function roster_show_subs_callback() { $this->print_checkbox('roster_show_subs'); echo '&nbsp;&nbsp;(show_subs)'; } 
    public function song_in_plan_callback() { $this->print_checkbox('song_in_plan'); echo '&nbsp;&nbsp;(in_plan)'; } 
    public function roster_departments_callback() { $this->print_input_text('text', 'roster_departments'); echo '&nbsp;&nbsp;(departments)'; } 
    public function songs_service_types_callback() { $this->print_input_text('text', 'songs_service_types'); echo '&nbsp;&nbsp;(service_types)'; } 
    public function songs_categories_callback() { $this->print_input_text('text', 'songs_categories'); echo '&nbsp;&nbsp;(categories)'; } 
    public function roster_callback($args) { $this->print_checkbox('roster_' . $args['field']); } 
	public function songs_callback($args) { $this->print_checkbox('songs_' . $args['field']); } 
	
}
?>